<?php
/**
 * http://new-dev.ru/
 * author GoldSoft <newdevexpert@gmail.com>
 * Copyright (c) New-Dev.ru
 */

namespace WebCash;

defined('DATALIFEENGINE') or exit('Access Denied');

class Gateways_Betatransfer extends AddonSettings
{
    const API_URL = 'https://betatransfer.io/api';
    protected $alias = 'betatransfer';

    public function renderPaymentForm($invoice_id, $cs_key, $gw_item_id) {
        if ($str = $this->verifyGatewayEnable())
            return $str;

        if (!$checkout_store = $this->checkout->getDataStore($cs_key))
            return '';

        $service_info = $this->getServiceInfo();

        if (!$gw_item = safe_array_access($service_info, 'items', $gw_item_id))
            trigger_error(basename(__FILE__).', line: '.__LINE__, E_USER_ERROR);

        $amount = $this->wc_currency->convertMainCurrencyTo($checkout_store['amount'], $this->currency);
        $payment_desc = ($this->webcash->site_url.' - '.to_utf8(safe_array_access($checkout_store, 'checkout_header')));

        $params = array(
            'amount' => number_format($amount, 2, '.', ''),
            'currency' =>  $this->currency,
            'orderId' => $invoice_id,
            'paymentSystem' => $this->payment_system,
            'urlResult' => $this->checkout->getGatewayProcessingUrl($this->alias),
            'urlSuccess' => $this->checkout->successCheckoutUrl($invoice_id, $cs_key),
            'urlFail' => $this->checkout->failCheckoutUrl($invoice_id, $cs_key),
            'locale' => $this->language,
            'redirect' => 1,
            'payerId' => $this->user->nonce,
            'payerEmail' => $this->helper->htmlspecialchars(safe_array_access($checkout_store, 'email')),
        );

        $params['sign'] = $this->signData($params);

        $tpl = $this->webcash->getTplInstance();
        $tpl->assign('invoice_id', $invoice_id);
        $tpl->assign('api_url', self::API_URL.'/payment?token='.$this->token);
        $tpl->assign('amount', $params['amount']);
        $tpl->assign('description', $this->helper->htmlspecialchars($payment_desc));
        $tpl->assign('email', $params['payerEmail']);
        $tpl->assign('params', $params);

        $tpl->assign('gw_item_id', $gw_item_id);
        $tpl->assign('gw_item', $gw_item);
        $tpl->assign('gateway_header', $service_info['name']);
        $tpl->assign('gw_alias', $this->alias);
        $tpl->assign('gateway_cfg', $this->getCfgPublicParams());
        $tpl->assign('addon_settings_link', $this->renderAddonSettingsLink());
        $tpl->assign('user_hash', $this->user->nonce);
        $tpl->load_template('/modules/webcash/gateways/betatransfer/checkout.tpl');

        $tpl->compile('content');

        return $tpl->result['content'];
    }

    public function signData($params) {
        if (!empty($params['sign'])) {
            unset($params['sign']);
        }

        array_push($params, $this->secret_key);
        return md5(implode('', $params));
    }

    public function processing() {
        // проверка на наличие обязательных полей
        foreach(array(
                    'amount',
                    'currency',
                    'orderId',
                    'sign',
                ) as $field) {
            if (!POST($field)) {
                $this->printError('Не указаны обязательные данные');
            }
        }

        // нормализация данных
        $amount = only_float($_POST['amount']);
        $invoice_id = (int)$_POST['orderId'];

        // проверка значения сигнатуры
        $arr = array(
            $_POST['amount'],
            $_POST['orderId'],
        );
        $signature = $this->signData($arr);

        if ($_POST['sign'] != $signature) {
            $this->printError('Неверная подпись '.$_POST['sign']);
        }

        $this->readSettingsFromFile();

        if (!$invoice_row = $this->webcash->getRowById($this->webcash->gateway_invoices_table, $invoice_id)) {
            $this->printError('Нет такого инвойса');
        }

        if ($invoice_row['state']) {
            $this->printError('Инвойс уже оплачен');
        }

        if ($invoice_row['gateway'] != $this->alias) {
            $this->printError('Инвойс не той платежной системы');
        }

        if ($invoice_row['amount'] > $amount) {
            $this->printError('Неверная сумма: '.$amount);
        }

        $sender = safe_array_access($invoice_row, 'email') or $sender = $this->user->ip;
        $payment_id = $this->processAfterPayment($invoice_id, $sender);

        if ($payment_row = $this->webcash->getRowById($this->webcash->gateway_payments_table, $payment_id)) {
            $this->checkout->gatewaySuccessPayment($invoice_row, $payment_row);

            exit('OK');
        }

        echo 'error';

        $this->printError('Error '.__LINE__);
    }

    public function printError($text) {
        $text = 'Ошибка! '.$text;
        if (WebCash::DEBUG_SETTINGS) {
            fs_log('Merchant error ('.$this->alias.'): '.$text);
            echo $text;
        }

        exit;
    }

    public function processAfterPayment($invoice_id, $sender) {
        $gateway_details_arr = POST();
        $gateway_details_arr = array_map(array($this->helper, 'htmlspecialchars'), $gateway_details_arr);

        return $this->checkout->addPaymentToDb($invoice_id, $sender, $gateway_details_arr);
    }

    public function getServiceInfo() {
        $result = array(
            'name' => __('Betatransfer'),
            'alias' => $this->alias,
            'items' => array(
                1 => array(
                    'title' => __('Перейти к оплате'),
                    'image' => 'betatransfer.png',
                ),
            ),
        );

        return $result;
    }
}